
% /*=================================================================
%  * Feature extraction script for professional soccer commentaries.
%  * Publication:
%  * Understanding of Professional Soccer Commentaries. 
%  *    By H. Hajisjirzi, M. Rastegari, A. Farhadi, J. Hodgins
%  *    In Proceeding of UAI 2012
%  * 
%  * Code is writen by Mohammad Rastegari. Report any bugs to mrastegari@disneyresearch.com
%  * 
%  * Copyright (c) 2011
%  * Permission is hereby granted, free of charge, to any person obtaining a 
%  * copy of this software and associated documentation files (the “Dataset and Script”),  
%  * THE MATERIALS IS PROVIDED “AS IS”, WITHOUT WARRANTY. 
%  * 
%  * The calling syntax in matlab is:
%  *
%  *		[model]=game_feature_extractor(game_info,player_or_flag,event_or_flag, time_interval);
%  *
%  *Inputs:
%  * game_info      : is the result of "games_info_packing.m" 
%  * player_or_flag : a binary value {0,1} that sellect to OR the player features within a time span
%  * event_or_flag  : a binary value {0,1} that sellect to OR the event features within a time span
%  * time_interval  : is a 1-by-2 doubel matrix [-t1,t2], where t1 is teh begining time and t2 is end time for each bucket.
%  *
%  *Outputs:
%  * model: is a structure containing all features and data for learning ESVM.
%  *=================================================================*/

function [model]=game_feature_extractor(game_info,player_or_flag, event_or_flag, time_interval)

addpath(genpath('./'))
load word_list.mat;
load word_num.mat;

teamName1=game_info.teamName1;
teamName2=game_info.teamName2;
gameName=game_info.gameName;




players1=read_teamplayers(teamName1);
players2=read_teamplayers(teamName2);

%time_interval=[-50,90];
time_interval_player_hist_sent=[-2,0];
time_interval_event_hist_type=[-1,0];


game_text=word_extractor([gameName '.text'],'stopwords.txt','punctuations.txt', 'phrasal_alphabet.txt');
text_feat=creat_feature_for_sentences(game_text,word_list,players1,players2);
event_data=read_event_feat([gameName '.feat']);
event_text=read_event_text([gameName '.events']);
event_types=event_type_read('eventtypes.txt');
%%% creating positive samples
pos_samples=logical([]);
examplar_train_align_label=[];
examplar_classifiers=[];
M=read_align_file([gameName '.align']);
indexes=[];
train_accuracy=[];
pos_label=[];
neg_data=[];



count=0;

[sw sw_idx]=sort(word_num,'descend');
text_feat.word_hist=text_feat.word_hist(sw_idx(1:100),:);
sorted_word_list=word_list(sw_idx(1:100));

for i=1:length(game_text.time)
    player_hist_sent(:,i)=creating_player_hist(players1,players2,game_text.sentence{i});
    
end
for j=1:length(event_text)
    
    player_hist_event(:,j)=creating_player_hist_event(players1,players2,event_text{j});
end

D_sent=dist2([text_feat.word_hist' player_hist_sent'],[text_feat.word_hist' player_hist_sent']);
[h k]=find(D_sent==0);
for i=1:length(h)
    D_sent(h(i),k(i))=NaN;
end

D_event=dist2([event_data.type_hist player_hist_event'],[event_data.type_hist player_hist_event']);
[h k]=find(D_event==0);
for i=1:length(h)
    D_event(h(i),k(i))=NaN;
end


%%%% ORing features along time 
if player_or_flag
   player_hist_event=bin_hist_or(player_hist_event,time_interval_player_hist_sent);
end
if event_or_flag
   event_data.type_hist=bin_hist_or(event_data.type_hist',time_interval_event_hist_type)';
end


for i=1:length(game_text.time)
    i
    block_idx{i}=[];
    M{i}=M{i}+1;
    dt=(event_data.time-game_text.time(i));
    idx=find((dt>=time_interval(1))&(dt<=time_interval(2)));
    
    %% packing up data for examplar training
    examplar_train_data=[];
    i
    for j=1:length(idx)
        indexes=[indexes; [i idx(j)] ];
        count=count+1;
        block_idx{i}=[block_idx{i} ; count];
        
        event_matches(count,:)=event_match(event_types,event_text{idx(j)},game_text.sentence{i});
        match=strcmpi(text_feat.playerName{i},event_data.player{idx(j)}(2:end-1));
        %pos_samples=[pos_samples [text_feat.nameExistance(i)==1; match; text_feat.word_hist(:,i)==1; event_data.type_hist(idx(j),:)'==1; event_matches(count,:)']];
        %pos_samples=[pos_samples [text_feat.nameExistance(i)==1; match; event_data.type_hist(idx(j),:)'==1 ]];
        examplar_train_data=[examplar_train_data [text_feat.nameExistance(i)==1; text_feat.word_hist(:,i)==1; [event_data.type_hist(idx(j),:)'==1; player_hist_sent(:,i); player_hist_event(:,idx(j));] ]];%  event_matches(count,:)'
        if ~isempty(find(idx(j)==M{i}(2:end)))
            examplar_train_align_label=[examplar_train_align_label; 1];
        else
            examplar_train_align_label=[examplar_train_align_label; -1];
        end
        [S idx_sent_near]=sort(D_sent(i,:));
        [S idx_sent_far]=sort(-D_sent(i,:));
        [S idx_event_near]=sort(D_event(idx(j),:));
        [S idx_event_far]=sort(-D_event(idx(j),:));
        examplar_negatives{count}=[];
        count2=0;
        for k=1:10
            for h=1:10
                count2=count2+1;
                iNeg=idx_sent_near(k);
                jNeg=idx_event_far(h);
                match=strcmpi(text_feat.playerName{iNeg},event_data.player{jNeg}(2:end-1));
                examplar_negatives{count}=[examplar_negatives{count} [text_feat.nameExistance(iNeg)==1; text_feat.word_hist(:,iNeg)==1; [event_data.type_hist(jNeg,:)'==1; player_hist_sent(:,iNeg); player_hist_event(:,jNeg);]]];
                negative_indexes{count}(count2,:)=[iNeg jNeg];
                
                count2=count2+1;
                iNeg=idx_sent_far(k);
                jNeg=idx_event_near(h);
                match=strcmpi(text_feat.playerName{iNeg},event_data.player{jNeg}(2:end-1));
                examplar_negatives{count}=[examplar_negatives{count} [text_feat.nameExistance(iNeg)==1; text_feat.word_hist(:,iNeg)==1; [event_data.type_hist(jNeg,:)'==1; player_hist_sent(:,iNeg); player_hist_event(:,jNeg);]]];
                
                negative_indexes{count}(count2,:)=[iNeg jNeg];
            end
        end
        examplar_negatives{count}=[examplar_negatives{count}(1:174,:); player_match(examplar_negatives{count})];
    end
    
    pos_samples=[pos_samples [examplar_train_data(1:174,:); player_match(examplar_train_data)]];
    
    
%     partial_neg=cell2mat(examplar_negatives);
%     n_par=length(partial_neg(1,:));
%     neg_data=[neg_data partial_neg(:,round([1:n_par/40:n_par]))];
    
    
end


model.pos_samples=pos_samples;
model.examplar_negatives=examplar_negatives;
model.examplar_train_align_label=examplar_train_align_label;
model.indexes=indexes;
model.game_text=game_text;
model.event_text=event_text;
model.word_list=sorted_word_list;


